#include <cmath>
#include <ctime>
#include <vector>
#include <iterator>


#ifndef _MAFIN_H
#define _MAFIN_H

typedef unsigned char boolean;

#define false 0
#define true 1

#define PI 3.14159265358979
#define DEUXPI 6.283185306
#define PI2 1.570796326794

#define PRECISION 1.0e-7 /*Precision for the bivariate Gaussian cumulative distribution function*/
#define PRICE_ACCURACY 1e-5 /*Precision for the implied volatility search*/
#define FAC_SLOPE 0.01


#define MAXLOOPS 5000

#define MAX(A,B) ( (A) > (B) ? (A):(B) )
#define MIN(A,B) ( (A) < (B) ? (A):(B) )
#define SQR(X) ((X)*(X))

double nd(double x);
double N(double x);


/*numerical inverse of the cumulative Gaussian distribution function*/
double Inverse_erf_Moro(double x);
void tsb2_seq(int *n, float x[]);

int Call_BlackScholes_73(double s, double Maturity, double Strike, double r,double divid, double sigma, double *ptprice, double *ptdelta, double *ptgamma);

int Put_BlackScholes_73(double s, double Maturity, double Strike, double r,double divid, double sigma, double *ptprice, double *ptdelta, double *ptgamma);

int ImpliedVol(double price,double s0,double Maturity, double Strike,double eqr, double eqdiv,double sigma_ref,double *volimpl, double *impliedDelta, double *impliedGamma);

void Progonka_in_place(const std::vector<double>& low, const std::vector<double>& diag, const
 std::vector<double>& up, std::vector<double>& rightside_result) ;

void Progonka( std::vector<double>::const_iterator low_begin,  
			  std::vector<double>::const_iterator diag_begin,  std::vector<double>::const_iterator up_begin,
               std::vector<double>::const_iterator rs_begin,  
			   std::vector<double>::iterator result_begin,  std::vector<double>::iterator result_end);

enum Vanilla {CallEuro=1,PutEuro=2,CallAmer=3,PutAmer=4};

///================== P R I C E === N O N E ==============================================================================

int TreePriceNone(double s0, Vanilla Type, double Maturity,  double Strike, double r,double  divid, double sigma, double *ptprice, double *ptdelta, double *ptgamma, int nTime, double lambda);


int  ImplPriceNone(double s0, Vanilla Type, double Maturity,  double Strike, double r,double  divid, double sigma, double *ptprice, double *ptdelta, double *ptgamma, int nTime,int nSpace);

int  CNPriceNone(double s0, Vanilla Type, double Maturity,  double Strike, double r,double  divid, double sigma, double *ptprice, double *ptdelta, double *ptgamma, int nTime,int nSpace);

///===================== U P === A N D === O U T ============================================================
int TreePriceUpOut(double s0,  Vanilla Type, double Maturity,  double Strike, double barrier,  double rebate, double r,double  divid, double sigma, double *ptprice, double *ptdelta, double *ptgamma, int nTime, double lambda);

int  ImplPriceUpOut(double s0,  Vanilla Type, double Maturity,  double Strike, double barrier,  double rebate, double r,double  divid, double sigma, double *ptprice, double *ptdelta, double *ptgamma, int nTime,int nSpace, double lambda);


#endif
